'use strict';

var Logs = function(){
    var vars = {
            $eventLog : $(".js-event-log"),
            $dataLog : $(".js-data-log"),
            $upsHistory : $(".js-ups-history"),
            $eventLogTable : {}, //store event log table session for destroying it
            $dataLogTable : {}, //store data log table session for destroying it
            dbg: true
        },

        loadDataLog = function loadDataLog(data){
            //loading
            vars.$dataLog.find(".js-log-loading").removeClass('hide');
            vars.$dataLog.find(".js-log-table").addClass('hide');

            fetchLog(Config.get("dataLogSource"), data)
                .done(function(res){
                    if(res.response==="ok"){
                        setTimeout(function(){
                            var table = vars.$dataLog.find(".js-log-table table");

                            if ( $.fn.dataTable.isDataTable( table ) ) {
                                vars.$dataLogTable.destroy();
                            }

                            vars.$dataLogTable = table.DataTable( {
                                "data": res.data,
                                "columnDefs": [
                                    { className: "dt-body-right", "targets": [ 1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18 ] }
                                ],
                                "columns":[
                                    { "data": "timedate" },
                                    { "data": "vin1min" },
                                    { "data": "vin2min" },
                                    { "data": "vin3min" },
                                    { "data": "finmed",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "vbyp1med" },
                                    { "data": "vbyp2med" },
                                    { "data": "vbyp3med" },
                                    { "data": "fbypmed",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "aout1max",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "aout2max",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "aout3max",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "load1max" },
                                    { "data": "load2max" },
                                    { "data": "load3max" },
                                    { "data": "tsysmed",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "tmaxmax",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(1);
                                        }
                                    },
                                    { "data": "bwork",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(2);
                                        }
                                    },
                                    { "data": "ldonbyp",
                                        "render" : function( data, type, full ) {
                                            return parseFloat(data).toFixed(2);
                                        }
                                    }
                                ],
                                deferRender:    true,
                                scrollY:        500,
                                scrollX:        true,
                                scrollCollapse: true,
                                scroller:       true,
                                processing:     true,
                                bInfo :         false,
                                ordering:       false,
                                searching:      false
                            } );
                        }, 100);

                    }
                })
                .always(function(){
                    vars.$dataLog.find(".js-log-loading").addClass('hide');
                    vars.$dataLog.find(".js-log-table").removeClass('hide');
                });
        },

        downloadDataLog = function downloadDataLog(data){
            window.location.href = Config.get("dataLogDownloadSource")+"?"+$.param(data);
            //fetchLog(Config.get("dataLogDownloadSource"), data);
        },

        loadEventLog = function loadEventLog(data){

            //loading
            vars.$eventLog.find(".js-log-loading").removeClass('hide');
            vars.$eventLog.find(".js-log-table").addClass('hide');

            fetchLog(Config.get("eventLogSource"), data)
                .done(function(res){
                    if(res.response==="ok"){
                        setTimeout(function(){
                            var table = vars.$eventLog.find(".js-log-table table");

                            if ( $.fn.dataTable.isDataTable( table ) ) {
                                vars.$eventLogTable.destroy();
                            }

                            var i;
                            for( i=0; i<res.data.length; i++ ) {
                                var type = res.data[i].type;
                                if( type === "Rise") {
                                    res.data[i].type = '<img style="max-height: 16px;max-width: 48px" src="assets/images/ico-EventLog-rise.png">';
                                }
                                else if( type === "Fall") {
                                    res.data[i].type = '<img style="max-height: 16px;max-width: 48px" src="assets/images/ico-EventLog-fall.png">';
                                }
                                else {
                                    res.data[i].type = '<img style="max-height: 16px;max-width: 48px" src="assets/images/ico-EventLog-none.png">';
                                }
                            }

                            vars.$eventLogTable = table.DataTable( {
                                "data": res.data,
                                "columns": [
                                    { 'data': 'timedate'},
                                    { 'data': 'codeStr'},
                                    { 'data': 'type'},
                                    { 'data': 'category'}
                                ],
                                deferRender:    true,
                                scrollY:        500,
                                scrollCollapse: true,
                                scroller:       true,
                                processing:     true,
                                bInfo :         false,
                                ordering:       false,
                                searching:      false
                            } );
                        }, 100);

                    }
                })
                .always(function(){
                    vars.$eventLog.find(".js-log-loading").addClass('hide');
                    vars.$eventLog.find(".js-log-table").removeClass('hide');
                });
        },

        downloadEventLog = function downloadEventLog(data){
            window.location.href = Config.get("eventLogDownloadSource")+"?"+$.param(data);
            //fetchLog(Config.get("eventLogDownloadSource"));
        },

        downloadUpsHistory = function downloadUpsHistory(){
            $.ajax({
                url: 'cgi-bin/log_check.cgi',
                method : 'get',
                dataType : 'json'
            }).done(function(res){
                if(res.response === "ok"){
                    window.location.href = Config.get("upsHistoryDownloadSource");
                }else{
                    showError(res.message);
                }
            });
        },

        fetchLog = function fetchLog(url, data){
            if(url === undefined){ url = 'json/testdata.json'; }
            if(data === undefined){ data = {}; }
            return $.ajax({url: url, method : 'get', data: data, dataType : 'json'});
        },

    // ----- Init -----
        initUpsHistory = function initUpsHistory(){

            $.ajax({
                url: 'cgi-bin/log_check.cgi',
                method : 'get',
                dataType : 'json'
            }).done(function(res){
                if(res.response === "ok"){
                    $(".js-ups-history-tab").removeClass("hidden");
                }
            });


            var $form = vars.$upsHistory.find(".js-ups-history-form");
            $form.on("submit", function(e){e.preventDefault();});

            $form.validate({
                submitHandler : function(form){
                    downloadUpsHistory();
                }
            });
        },

        initEventLog = function initEventLog(){
            //loadEventLog();

            //vars.$eventLog.find(".js-download-btn").on("click", function(e){
            //    e.preventDefault();
            //    downloadEventLog();
            //});

            var $form = vars.$eventLog.find(".js-log-form");

            vars.$eventLog.find(".js-log-table").addClass('hide');
            vars.$eventLog.find(".js-log-loading").addClass('hide');
            vars.$eventLog.find("#date_start").val(moment(new Date()).subtract(1, 'months').format("DD/MM/YYYY HH:mm"));
            vars.$eventLog.find("#date_end").val(moment(new Date()).add(1, 'minutes').format("DD/MM/YYYY HH:mm"));

            $form.on("submit", function(e){e.preventDefault();});

            $form.validate({
                submitHandler : function(form){
                    form = $(form);

                    var data = _.chain(form.serializeArray({ checkboxesAsBools: true }))
                        .indexBy('name')
                        .mapValues('value')
                        .value();

                    var ds = moment(data.date_start, "DD/MM/YYYY HH:mm").format("X");
                    var de = moment(data.date_end, "DD/MM/YYYY HH:mm").format("X");

                    //swap dates if end > start
                    if (ds <= de) {
                        data.date_start = ds;
                        data.date_end = de;
                    } else {
                        data.date_start = de;
                        data.date_end = ds;
                    }

                    if(data.submitType === "log"){
                        delete data.submitType;
                        loadEventLog(data);
                    }
                    else if(data.submitType === "download"){
                        delete data.submitType;
                        downloadEventLog(data);
                    }
                }
            });
        },

        initDataLog = function initDataLog(){
            var $form = vars.$dataLog.find(".js-log-form");

            vars.$dataLog.find(".js-log-table").addClass('hide');
            vars.$dataLog.find(".js-log-loading").addClass('hide');
            vars.$dataLog.find("#datalog-date_start").val(moment(new Date()).subtract(1, 'week').format("DD/MM/YYYY HH:mm"));
            vars.$dataLog.find("#datalog-date_end").val(moment(new Date()).add(1, 'minutes').format("DD/MM/YYYY HH:mm"));

            $form.on("submit", function(e){e.preventDefault();});

            $form.validate({
                submitHandler : function(form){
                    form = $(form);

                    var data = _.chain(form.serializeArray({ checkboxesAsBools: true }))
                        .indexBy('name')
                        .mapValues('value')
                        .value();

                    var ds = moment(data.date_start, "DD/MM/YYYY HH:mm").format("X");
                    var de = moment(data.date_end, "DD/MM/YYYY HH:mm").format("X");

                    //swap dates if end > start
                    if (ds <= de) {
                        data.date_start = ds;
                        data.date_end = de;
                    } else {
                        data.date_start = de;
                        data.date_end = ds;
                    }

                    if(data.submitType == "log"){
                        delete data.submitType;
                        loadDataLog(data);
                    }
                    else if(data.submitType == "download"){
                        delete data.submitType;
                        downloadDataLog(data);
                    }
                }
            });

        },

        init = function init() {
            initEventLog();
            initDataLog();
            initUpsHistory();

            $('a[data-toggle="tab"]').on('shown.bs.tab', function(){
                $.each($.fn.dataTable.tables(true), function(){
                    $(this).DataTable().scroller.measure();
                });
            });
        };

    return {
        init: init
    }
}();
